/*
 * Author: G. Monz (DK7IO), 2022-06-20
 * This file is distributed without any warranty.
 * */

using System;
using System.Windows.Forms;
using GM.Geodesy;

namespace QthManager
{
	public partial class MainForm
	{
		readonly ContextMenuStrip SaveCsvContextMenu1 = new ContextMenuStrip();
		readonly ContextMenuStrip SaveKmlContextMenu1 = new ContextMenuStrip();

		public GeographicalPoint GeographicalPoint1 { get; set; }

		void ButtonSaveCsv1_MouseUp(object sender, MouseEventArgs e)
		{
			if (e.Button == MouseButtons.Left)
			{
				ResetState1();

				try
				{
					Application.DoEvents();
					SetBusyCursor();
					SaveCsv(TextBoxLocator1.Text, TextBoxCurrentLocator1, openFile: true);
				}
				catch (Exception ex)
				{
					SetState1(ex);
				}
			}
			else if (e.Button == MouseButtons.Right)
			{
				ResetState1();

				try
				{
					ShowSaveCsvContextMenu(e, ButtonSaveCsv1, SaveCsvContextMenu1, TextBoxLocator1.Text);
				}
				catch (Exception ex)
				{
					SetState1(ex);
				}
			}
		}

		void SaveCsvContextMenu1_ItemClicked(object sender, ToolStripItemClickedEventArgs e)
		{
			ResetState1();

			try
			{
				SaveCsvContextMenu1.Close(ToolStripDropDownCloseReason.ItemClicked);
				SaveCsvContextMenu_ItemClicked(e, TextBoxCurrentLocator1);
			}
			catch (Exception ex)
			{
				SetState1(ex);
			}
		}

		void ButtonSaveKml1_MouseUp(object sender, MouseEventArgs e)
		{
			if (e.Button == MouseButtons.Left)
			{
				ResetState1();

				try
				{
					Application.DoEvents();
					SetBusyCursor();
					SaveKml(TextBoxLocator1.Text, TextBoxCurrentLocator1, openFile: true);
				}
				catch (Exception ex)
				{
					SetState1(ex);
				}
			}
			else if (e.Button == MouseButtons.Right)
			{
				ResetState1();

				try
				{
					ShowSaveKmlContextMenu(e, ButtonSaveKml1, SaveKmlContextMenu1, TextBoxLocator1.Text);
				}
				catch (Exception ex)
				{
					SetState1(ex);
				}
			}
		}

		void SaveKmlContextMenu1_ItemClicked(object sender, ToolStripItemClickedEventArgs e)
		{
			ResetState1();

			try
			{
				SaveKmlContextMenu1.Close(ToolStripDropDownCloseReason.ItemClicked);
				SaveKmlContextMenu_ItemClicked(e, TextBoxCurrentLocator1);
			}
			catch (Exception ex)
			{
				SetState1(ex);
			}
		}

		void ButtonLocatorEnter1_Click(object sender, EventArgs e)
		{
			ResetState1();

			try
			{
				Locator1Enter(true);
			}
			catch (Exception ex)
			{
				SetState1(ex);
			}
		}

		void TextBoxLocator1_TextChanged(object sender, EventArgs e)
		{
			LocatorTextBoxChanged(TextBoxLocator1.Text, ButtonLocatorEnter1,
				ButtonSaveKml1, ButtonSaveCsv1, ButtonWaypoints1);
		}

		void LinkLabelGeoHack1_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
		{
			try
			{
				if (e.Button == MouseButtons.Left)
					StartProcess(LinkLabelGeoHack1.Text);
			}
			catch (Exception ex)
			{
				SetState1(ex);
			}
		}

		void TextBoxLocatorState1_DoubleClick(object sender, EventArgs e) => ShowException(TextBoxLocatorState1);

		void ButtonLocatorPrecisionEnter1_Click(object sender, EventArgs e) => ButtonLocatorPrecisionEnter(ButtonLocatorPrecisionEnter1, TextBoxLocator1);

		void ListBoxLocatorPrecisions1_SelectedIndexChanged(object sender, EventArgs e) => ListBoxLocatorPrecisionsSelectedIndexChanged(ListBoxLocatorPrecisions1, ButtonLocatorPrecisionEnter1);

		void ButtonDEnter1_Click(object sender, EventArgs e)
		{
			if (!D1Enter())
				return;

			try
			{
				Locator1Enter(false);
			}
			catch (Exception ex)
			{
				SetState1(ex);
			}
		}

		void ButtonDMEnter1_Click(object sender, EventArgs e)
		{
			if (!DM1Enter())
				return;

			try
			{
				Locator1Enter(false);
			}
			catch (Exception ex)
			{
				SetState1(ex);
			}
		}

		void ButtonDMSEnter1_Click(object sender, EventArgs e)
		{
			if (!DMS1Enter())
				return;

			try
			{
				Locator1Enter(false);
			}
			catch (Exception ex)
			{
				SetState1(ex);
			}
		}

		void ButtonWaypoints1_Click(object sender, EventArgs e)
		{
			ResetState1();

			try
			{
				var geographicalPoint = GeographicalPoint1;
				if (geographicalPoint == null)
					throw new InvalidOperationException("No locator or position applied.");

				SetBusyCursor();

				string name = geographicalPoint.Name;
				{
					SaveWaypoints(geographicalPoint, TextBoxLocator1.Text,
						textBoxWaypointsAzimuth1, textBoxWaypointsCount1, textBoxWaypointsKm1);
				}
				geographicalPoint.Name = name;
			}
			catch (Exception ex)
			{
				SetState1(ex);
			}
		}

		void GeographicalPoint1Updated()
		{
			GeographicalPointUpdated(GeographicalPoint1,
				textBoxDND1, textBoxDED1,
				textBoxDMND1, textBoxDMNM1, textBoxDMED1, textBoxDMEM1,
				textBoxDMSND1, textBoxDMSNM1, textBoxDMSNS1, textBoxDMSED1, textBoxDMSEM1, textBoxDMSES1);
			SetGeoHackLink(LinkLabelGeoHack1, labelGeoHack1, GeographicalPoint1);
		}

		void Locator1Enter(bool handleGeographicalPoint)
		{
			var items = ListBoxLocatorPrecisions1.Items;
			items.Clear();

			string locator = TextBoxLocator1.Text.Trim();

			var positionInRectangle = LocatorPositionInGrid(groupBoxLocatorPositionInGrid1);
			GeographicalPoint1 = new GeographicalPoint(locator, positionInRectangle);

			var maidenheadLocators = GeographicalPoint1.GetMaidenheadLocators(CheckBoxSmallLettersForSubsquares.Checked);

			foreach (string maidenheadLocator in maidenheadLocators)
				items.Add(maidenheadLocator);

			ListBoxLocatorPrecisions1.SelectedIndex = ListBoxLocatorPrecisionsIndex(locator);

			if (handleGeographicalPoint)
				GeographicalPoint1Updated();
		}

		bool D1Enter()
		{
			bool result = true;

			ResetGeographicalCoordinates1();
			textBoxDMND1.Text = textBoxDMNM1.Text = textBoxDMED1.Text = textBoxDMEM1.Text = null;
			textBoxDMSND1.Text = textBoxDMSNM1.Text = textBoxDMSNS1.Text = textBoxDMSED1.Text = textBoxDMSEM1.Text = textBoxDMSES1.Text = null;
			ResetState1();

			string latitudeD = NullValue(textBoxDND1);
			string longitudeD = NullValue(textBoxDED1);

			try
			{
				double latitude = double.Parse(latitudeD);
				double longitude = double.Parse(longitudeD);

				GeographicalPoint1 = new GeographicalPoint(latitude, longitude);
				GeographicalPoint1Updated();

				TextBoxLocator1.Text = GeographicalPoint1.GetMaidenheadLocator(true, MaidenheadLocator.MaxPrecision);
			}
			catch (Exception ex)
			{
				result = false;
				SetState1(ex);
			}

			return result;
		}

		bool DM1Enter()
		{
			bool result = true;

			ResetGeographicalCoordinates1();
			textBoxDND1.Text = textBoxDED1.Text = null;
			textBoxDMSND1.Text = textBoxDMSNM1.Text = textBoxDMSNS1.Text = textBoxDMSED1.Text = textBoxDMSEM1.Text = textBoxDMSES1.Text = null;
			ResetState1();

			string latitudeD = NullValue(textBoxDMND1);
			string longitudeD = NullValue(textBoxDMED1);

			string latitudeM = NullValue(textBoxDMNM1);
			string longitudeM = NullValue(textBoxDMEM1);

			try
			{
				int latitudeDegrees = int.Parse(latitudeD);
				int longitudeDegrees = int.Parse(longitudeD);

				bool latitudeIsNegative = (latitudeDegrees < 0) || IsNegativeValue(latitudeD);
				bool longitudeIsNegative = (longitudeDegrees < 0) || IsNegativeValue(longitudeD);

				GeographicalPoint1 = new GeographicalPoint(
					latitudeIsNegative, latitudeDegrees, double.Parse(latitudeM),
					longitudeIsNegative, longitudeDegrees, double.Parse(longitudeM));
				GeographicalPoint1Updated();

				TextBoxLocator1.Text = GeographicalPoint1.GetMaidenheadLocator(true, MaidenheadLocator.MaxPrecision);
			}
			catch (Exception ex)
			{
				result = false;
				SetState1(ex);
			}

			return result;
		}

		bool DMS1Enter()
		{
			bool result = true;

			ResetGeographicalCoordinates1();
			textBoxDND1.Text = textBoxDED1.Text = null;
			textBoxDMND1.Text = textBoxDMNM1.Text = textBoxDMED1.Text = textBoxDMEM1.Text = null;
			ResetState1();

			string latitudeD = NullValue(textBoxDMSND1);
			string longitudeD = NullValue(textBoxDMSED1);

			string latitudeM = NullValue(textBoxDMSNM1);
			string longitudeM = NullValue(textBoxDMSEM1);

			string latitudeS = NullValue(textBoxDMSNS1);
			string longitudeS = NullValue(textBoxDMSES1);

			try
			{
				int latitudeDegrees = int.Parse(latitudeD);
				int longitudeDegrees = int.Parse(longitudeD);

				bool latitudeIsNegative = (latitudeDegrees < 0) || IsNegativeValue(latitudeD);
				bool longitudeIsNegative = (longitudeDegrees < 0) || IsNegativeValue(longitudeD);

				GeographicalPoint1 = new GeographicalPoint(
					latitudeIsNegative, latitudeDegrees, int.Parse(latitudeM), double.Parse(latitudeS),
					longitudeIsNegative, longitudeDegrees, int.Parse(longitudeM), double.Parse(longitudeS));
				GeographicalPoint1Updated();

				TextBoxLocator1.Text = GeographicalPoint1.GetMaidenheadLocator(true, MaidenheadLocator.MaxPrecision);
			}
			catch (Exception ex)
			{
				result = false;
				SetState1(ex);
			}

			return result;
		}

		void ResetGeographicalCoordinates1()
		{
			GeographicalPoint1 = null;
			ResetLocator(TextBoxLocator1, ListBoxLocatorPrecisions1);
		}

		void SetState1(Exception exception) => SetState(TextBoxLocatorState1, exception);

		void ResetState1() => ResetState(TextBoxLocatorState1, LinkLabelGeoHack1, labelGeoHack1, TextBoxCurrentLocator1);
	}
}