/*
 * Author: G. Monz (DK7IO), 2022-06-20
 * This file is distributed without any warranty.
 * */

using System;
using System.Windows.Forms;
using GM.Geodesy;

namespace QthManager
{
	public partial class MainForm
	{
		readonly ContextMenuStrip SaveCsvContextMenu2 = new ContextMenuStrip();
		readonly ContextMenuStrip SaveKmlContextMenu2 = new ContextMenuStrip();

		public GeographicalPoint GeographicalPoint2 { get; set; }

		void ButtonSaveCsv2_MouseUp(object sender, MouseEventArgs e)
		{
			if (e.Button == MouseButtons.Left)
			{
				ResetState2();

				try
				{
					Application.DoEvents();
					SetBusyCursor();
					SaveCsv(TextBoxLocator2.Text, TextBoxCurrentLocator2, openFile: true);
				}
				catch (Exception ex)
				{
					SetState2(ex);
				}
			}
			else if (e.Button == MouseButtons.Right)
			{
				ResetState2();

				try
				{
					ShowSaveCsvContextMenu(e, ButtonSaveCsv2, SaveCsvContextMenu2, TextBoxLocator2.Text);
				}
				catch (Exception ex)
				{
					SetState2(ex);
				}
			}
		}

		void SaveCsvContextMenu2_ItemClicked(object sender, ToolStripItemClickedEventArgs e)
		{
			ResetState2();

			try
			{
				SaveCsvContextMenu2.Close(ToolStripDropDownCloseReason.ItemClicked);
				SaveCsvContextMenu_ItemClicked(e, TextBoxCurrentLocator2);
			}
			catch (Exception ex)
			{
				SetState2(ex);
			}
		}

		void ButtonSaveKml2_MouseUp(object sender, MouseEventArgs e)
		{
			if (e.Button == MouseButtons.Left)
			{
				ResetState2();

				try
				{
					Application.DoEvents();
					SetBusyCursor();
					SaveKml(TextBoxLocator2.Text, TextBoxCurrentLocator2, openFile: true);
				}
				catch (Exception ex)
				{
					SetState2(ex);
				}
			}
			else if (e.Button == MouseButtons.Right)
			{
				ResetState2();

				try
				{
					ShowSaveKmlContextMenu(e, ButtonSaveKml2, SaveKmlContextMenu2, TextBoxLocator2.Text);
				}
				catch (Exception ex)
				{
					SetState2(ex);
				}
			}
		}

		void SaveKmlContextMenu2_ItemClicked(object sender, ToolStripItemClickedEventArgs e)
		{
			ResetState2();

			try
			{
				SaveKmlContextMenu2.Close(ToolStripDropDownCloseReason.ItemClicked);
				SaveKmlContextMenu_ItemClicked(e, TextBoxCurrentLocator2);
			}
			catch (Exception ex)
			{
				SetState2(ex);
			}
		}

		void ButtonLocatorEnter2_Click(object sender, EventArgs e)
		{
			ResetState2();

			try
			{
				Locator2Enter(true);
			}
			catch (Exception ex)
			{
				SetState2(ex);
			}
		}

		void TextBoxLocator2_TextChanged(object sender, EventArgs e)
		{
			LocatorTextBoxChanged(TextBoxLocator2.Text, ButtonLocatorEnter2,
				ButtonSaveKml2, ButtonSaveCsv2, ButtonWaypoints2);
		}

		void LinkLabelGeoHack2_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
		{
			try
			{
				if (e.Button == MouseButtons.Left)
					StartProcess(LinkLabelGeoHack2.Text);
			}
			catch (Exception ex)
			{
				SetState2(ex);
			}
		}

		void TextBoxLocatorState2_DoubleClick(object sender, EventArgs e) => ShowException(TextBoxLocatorState2);

		void ButtonLocatorPrecisionEnter2_Click(object sender, EventArgs e) => ButtonLocatorPrecisionEnter(ButtonLocatorPrecisionEnter2, TextBoxLocator2);

		void ListBoxLocatorPrecisions2_SelectedIndexChanged(object sender, EventArgs e) => ListBoxLocatorPrecisionsSelectedIndexChanged(ListBoxLocatorPrecisions2, ButtonLocatorPrecisionEnter2);

		void ButtonDEnter2_Click(object sender, EventArgs e)
		{
			if (!D2Enter())
				return;

			try
			{
				Locator2Enter(false);
			}
			catch (Exception ex)
			{
				SetState2(ex);
			}
		}

		void ButtonDMEnter2_Click(object sender, EventArgs e)
		{
			if (!DM2Enter())
				return;

			try
			{
				Locator2Enter(false);
			}
			catch (Exception ex)
			{
				SetState2(ex);
			}
		}

		void ButtonDMSEnter2_Click(object sender, EventArgs e)
		{
			if (!DMS2Enter())
				return;

			try
			{
				Locator2Enter(false);
			}
			catch (Exception ex)
			{
				SetState2(ex);
			}
		}

		void ButtonWaypoints2_Click(object sender, EventArgs e)
		{
			ResetState2();

			try
			{
				var geographicalPoint = GeographicalPoint2;
				if (geographicalPoint == null)
					throw new InvalidOperationException("No locator or position applied.");

				SetBusyCursor();

				string name = geographicalPoint.Name;
				{
					SaveWaypoints(geographicalPoint, TextBoxLocator2.Text,
						textBoxWaypointsAzimuth2, textBoxWaypointsCount2, textBoxWaypointsKm2);
				}
				geographicalPoint.Name = name;
			}
			catch (Exception ex)
			{
				SetState2(ex);
			}
		}

		void GeographicalPoint2Updated()
		{
			GeographicalPointUpdated(GeographicalPoint2,
				textBoxDND2, textBoxDED2,
				textBoxDMND2, textBoxDMNM2, textBoxDMED2, textBoxDMEM2,
				textBoxDMSND2, textBoxDMSNM2, textBoxDMSNS2, textBoxDMSED2, textBoxDMSEM2, textBoxDMSES2);
			SetGeoHackLink(LinkLabelGeoHack2, labelGeoHack2, GeographicalPoint2);
		}

		void Locator2Enter(bool handleGeographicalPoint)
		{
			var items = ListBoxLocatorPrecisions2.Items;
			items.Clear();

			string locator = TextBoxLocator2.Text.Trim();

			var positionInRectangle = LocatorPositionInGrid(groupBoxLocatorPositionInGrid2);
			GeographicalPoint2 = new GeographicalPoint(locator, positionInRectangle);

			var maidenheadLocators = GeographicalPoint2.GetMaidenheadLocators(CheckBoxSmallLettersForSubsquares.Checked);

			foreach (string maidenheadLocator in maidenheadLocators)
				items.Add(maidenheadLocator);

			ListBoxLocatorPrecisions2.SelectedIndex = ListBoxLocatorPrecisionsIndex(locator);

			if (handleGeographicalPoint)
				GeographicalPoint2Updated();
		}

		bool D2Enter()
		{
			bool result = true;

			ResetGeographicalCoordinates2();
			textBoxDMND2.Text = textBoxDMNM2.Text = textBoxDMED2.Text = textBoxDMEM2.Text = null;
			textBoxDMSND2.Text = textBoxDMSNM2.Text = textBoxDMSNS2.Text = textBoxDMSED2.Text = textBoxDMSEM2.Text = textBoxDMSES2.Text = null;
			ResetState2();

			string latitudeD = NullValue(textBoxDND2);
			string longitudeD = NullValue(textBoxDED2);

			try
			{
				double latitude = double.Parse(latitudeD);
				double longitude = double.Parse(longitudeD);

				GeographicalPoint2 = new GeographicalPoint(latitude, longitude);
				GeographicalPoint2Updated();

				TextBoxLocator2.Text = GeographicalPoint2.GetMaidenheadLocator(true, MaidenheadLocator.MaxPrecision);
			}
			catch (Exception ex)
			{
				result = false;
				SetState2(ex);
			}

			return result;
		}

		bool DM2Enter()
		{
			bool result = true;

			ResetGeographicalCoordinates2();
			textBoxDND2.Text = textBoxDED2.Text = null;
			textBoxDMSND2.Text = textBoxDMSNM2.Text = textBoxDMSNS2.Text = textBoxDMSED2.Text = textBoxDMSEM2.Text = textBoxDMSES2.Text = null;
			ResetState2();

			string latitudeD = NullValue(textBoxDMND2);
			string longitudeD = NullValue(textBoxDMED2);

			string latitudeM = NullValue(textBoxDMNM2);
			string longitudeM = NullValue(textBoxDMEM2);

			try
			{
				int latitudeDegrees = int.Parse(latitudeD);
				int longitudeDegrees = int.Parse(longitudeD);

				bool latitudeIsNegative = (latitudeDegrees < 0) || IsNegativeValue(latitudeD);
				bool longitudeIsNegative = (longitudeDegrees < 0) || IsNegativeValue(longitudeD);

				GeographicalPoint2 = new GeographicalPoint(
					latitudeIsNegative, latitudeDegrees, double.Parse(latitudeM),
					longitudeIsNegative, longitudeDegrees, double.Parse(longitudeM));
				GeographicalPoint2Updated();

				TextBoxLocator2.Text = GeographicalPoint2.GetMaidenheadLocator(true, MaidenheadLocator.MaxPrecision);
			}
			catch (Exception ex)
			{
				result = false;
				SetState2(ex);
			}

			return result;
		}

		bool DMS2Enter()
		{
			bool result = true;

			ResetGeographicalCoordinates2();
			textBoxDND2.Text = textBoxDED2.Text = null;
			textBoxDMND2.Text = textBoxDMNM2.Text = textBoxDMED2.Text = textBoxDMEM2.Text = null;
			ResetState2();

			string latitudeD = NullValue(textBoxDMSND2);
			string longitudeD = NullValue(textBoxDMSED2);

			string latitudeM = NullValue(textBoxDMSNM2);
			string longitudeM = NullValue(textBoxDMSEM2);

			string latitudeS = NullValue(textBoxDMSNS2);
			string longitudeS = NullValue(textBoxDMSES2);

			try
			{
				int latitudeDegrees = int.Parse(latitudeD);
				int longitudeDegrees = int.Parse(longitudeD);

				bool latitudeIsNegative = (latitudeDegrees < 0) || IsNegativeValue(latitudeD);
				bool longitudeIsNegative = (longitudeDegrees < 0) || IsNegativeValue(longitudeD);

				GeographicalPoint2 = new GeographicalPoint(
					latitudeIsNegative, latitudeDegrees, int.Parse(latitudeM), double.Parse(latitudeS),
					longitudeIsNegative, longitudeDegrees, int.Parse(longitudeM), double.Parse(longitudeS));
				GeographicalPoint2Updated();

				TextBoxLocator2.Text = GeographicalPoint2.GetMaidenheadLocator(true, MaidenheadLocator.MaxPrecision);
			}
			catch (Exception ex)
			{
				result = false;
				SetState2(ex);
			}

			return result;
		}

		void ResetGeographicalCoordinates2()
		{
			GeographicalPoint2 = null;
			ResetLocator(TextBoxLocator2, ListBoxLocatorPrecisions2);
		}

		void SetState2(Exception exception) => SetState(TextBoxLocatorState2, exception);

		void ResetState2() => ResetState(TextBoxLocatorState2, LinkLabelGeoHack2, labelGeoHack2, TextBoxCurrentLocator2);
	}
}