﻿/*
 * Author: G. Monz (DK7IO), 2022-06-20
 * This file is distributed without any warranty.
 * */

using System;
using System.Drawing;
using System.Drawing.Printing;
using System.Windows.Forms;

namespace QthManager
{
	public class Printer
	{
		Control ControlToPrint;

		public void Print(Control control, string documentName)
		{
			ControlToPrint = control;

			using (var printDocument = new PrintDocument())
			{
				printDocument.PrinterSettings = new PrinterSettings();

				printDocument.PrintPage += new PrintPageEventHandler(PrintDoc_PrintPage);
				{
					printDocument.DocumentName = documentName;
					printDocument.DefaultPageSettings.Margins = new Margins(50, 50, 50, 50);
					printDocument.DefaultPageSettings.Landscape = (ControlToPrint.Width > ControlToPrint.Height);
					printDocument.DefaultPageSettings.Color = printDocument.PrinterSettings.DefaultPageSettings.Color;
					printDocument.PrintController = new StandardPrintController();

					var printDialog = new PrintDialog
					{
						Document = printDocument,
						AllowSelection = true,
						AllowSomePages = true,
						UseEXDialog = true
					};

					if (printDialog.ShowDialog() == DialogResult.OK)
					{
						try
						{
							printDocument.Print();
						}
						catch (System.ComponentModel.Win32Exception)
						{
						}
					}
				}
				printDocument.PrintPage -= new PrintPageEventHandler(PrintDoc_PrintPage);
			}
		}

		void PrintDoc_PrintPage(object sender, PrintPageEventArgs e)
		{
			var bitmap = Bitmap(ControlToPrint);

			Rectangle destinationRectangle;
			int height = (bitmap.Height * e.MarginBounds.Width) / bitmap.Width;
			if (height > e.MarginBounds.Height)
			{
				destinationRectangle = new Rectangle(e.MarginBounds.X, e.MarginBounds.Y,
					(bitmap.Width * e.MarginBounds.Height) / bitmap.Height,
					e.MarginBounds.Height);
			}
			else
				destinationRectangle = new Rectangle(e.MarginBounds.X, e.MarginBounds.Y, e.MarginBounds.Width, height);

			e.Graphics.DrawImage(bitmap, destinationRectangle);
		}

		static Bitmap Bitmap(Control control)
		{
			if (control == null)
				throw new ArgumentNullException(nameof(control));

			int width = control.Width;
			int height = control.Height;
			var bitmap = new Bitmap(width, height);
			var targetBounds = new Rectangle(0, 0, width, height);
			control.DrawToBitmap(bitmap, targetBounds);
			return bitmap;
		}
	}
}