﻿/*
 * Author: G. Monz (DK7IO), 2011-10-28
 * This file is distributed without any warranty.
 * */

using System;
using GM.Geodesy;

namespace Test_GM.Geodesy
{
	class Program
	{
		static void Main(string[] args)
		{
			//Definition of a 'Geographical Point'
			{
				//Latitude = 0.0; Longitude = 0.0
				GeographicalPoint geographicalPoint1 = new GeographicalPoint();

				//Latitude = -20.0; Longitude = 35.12345
				GeographicalPoint geographicalPoint2 = new GeographicalPoint(-20, 35.12345);
				//Change the coordinates
				geographicalPoint2.Latitude = -21.54;
				geographicalPoint2.Longitude = 34;
				//Change the coordinates one more time
				geographicalPoint2.Latitude = geographicalPoint2.Latitude - 1;
				geographicalPoint2.Longitude -= 1;
				//Latitude = -22.54; Longitude = 33.0

				//Invalid latitude or longitude -> ArgumentException
				GeographicalPoint geographicalPoint2b = new GeographicalPoint(-100000, 35.12345);

				//Latitude = 55.7500; Longitude = 37.6158 (Moscow, Kremlin)
				GeographicalPoint geographicalPoint3a = new GeographicalPoint(
					false, 55, 45, 0,
					false, 37, 36, 57
				);

				//Latitude = 55.7500; Longitude = 37.6158
				//-> Negative numbers will ignored
				//-> For a sign use the flags only (first and fifth parameter, respectively)!
				GeographicalPoint geographicalPoint3b = new GeographicalPoint(
					false, -55, 45, 0,
					false, 37, -36, -57
				);

				//Utility function
				double longitude = Utilities.GetDegrees(false, 37, 36, 57);

				//Latitude = 55.7500; Longitude = 37.6158
				GeographicalPoint geographicalPoint3c = new GeographicalPoint(
					Utilities.GetDegrees(false, 55, 45, 0),
					longitude
				);

				//Invalid latitude or longitude -> ArgumentException
				GeographicalPoint geographicalPoint3d = new GeographicalPoint(
					false, 99, 45, 250,
					false, 37, 50000000, 57
				);

				//Latitude = 0.77777777777777779; Longitude = 11.333611111111111
				//Numbers exceed their common interval
				//-> Handle with care!
				GeographicalPoint geographicalPoint3e = new GeographicalPoint(
					false, 0, 45, 100,
					false, 3, 500, 1
				);

				//Latitude = -34.6042; Longitude = -58.3750 (NCDXF beacon LU4AA, Buenos Aires)
				GeographicalPoint geographicalPoint4a = new GeographicalPoint(
					"GF05TJ", //The 'Maidenhead Locator'
					PositionInRectangle.MiddleMiddle //The 'assumed' position in the square GF05TJ
				);

				//Invalid locator -> ArgumentException
				GeographicalPoint geographicalPoint4b = new GeographicalPoint(
					"ZZ",
					PositionInRectangle.BottomLeft
				);

				//Invalid locator -> ArgumentException
				GeographicalPoint geographicalPoint4c = new GeographicalPoint(
					"JN3",
					PositionInRectangle.BottomLeft
				);
			}

			//Distance and azimuth
			{
				GeographicalPoint northPole = new GeographicalPoint(
					false, 90, 0, 0, //Latitude
					false, 0, 0, 0 //Longitude: no matter (pole!)
				);
				GeographicalPoint southPole = new GeographicalPoint(
					true, 90, 0, 0, //Latitude
					false, 12, 34, 56 //Longitude: no matter (pole!)
				);

				double distance1 = northPole.GetDistance(southPole); //20063.928413300029

				double distance2 = northPole.GetDistance(
					new GeographicalPoint(true, 90, 0, 0, false, 0, 0, 0) //South Pole
				); //20015.089309294694

				double distance3 = northPole.GetDistance(northPole); //0.0

				string testLocator = "JO75ix07br"; //Transmitter mast 'Sölvesborg'

				double distance4 = GeographicalPoint.GetDistance(
					southPole,
					new GeographicalPoint(testLocator, PositionInRectangle.MiddleLeft)
				); //16233.409223743354

				double distance5 = northPole.GetDistance(testLocator, PositionInRectangle.MiddleLeft); //3781.68008555134

				double distance6 = GeographicalPoint.GetDistance(
					"AB34", PositionInRectangle.TopMiddle,
					testLocator, PositionInRectangle.MiddleLeft
				); //17876.164158852531

				double distance7 = GeographicalPoint.GetDistance(
					new GeographicalPoint(49, 0),
					new GeographicalPoint(50, 0)
				); //111.19494060718165

				double azimuthNorthToSouth = northPole.GetAzimuth(southPole); //173.70888888888888

				double azimuthSouthToNorth = southPole.GetAzimuth(northPole); //353.70888888888885

				double azimuthNorthToNorth = northPole.GetAzimuth(northPole); //0.0

				//6m Beacon LX0SIX
				GeographicalPoint LX0SIX = new GeographicalPoint("JN39AV", PositionInRectangle.MiddleMiddle);

				double azimuthBeaconLink1 = GeographicalPoint.GetAzimuth(
					new GeographicalPoint(+49.713, -2.203), //Alderney
					LX0SIX
				); //84.884387153645221

				double azimuthBeaconLink2 =
					new GeographicalPoint(+49.713, -2.203) //Alderney
					.GetAzimuth(LX0SIX)
				; //84.884387153645221

				double azimuthBeaconLink3 =
					new GeographicalPoint(+49.713, -2.203) //Alderney
					.GetAzimuth(
						new GeographicalPoint("JN39AV", PositionInRectangle.MiddleMiddle) //LX0SIX
					)
				; //84.884387153645221
			}

			//Formatted text output
			{
				GeographicalPoint geographicalPoint = new GeographicalPoint(-20.4987, 0.9976);
				bool forceAsciiCharacters = false; //Use special characters
				string separator = " / ";

				string sD0 = geographicalPoint.ToString(AngleFormat.sD, 0, separator, forceAsciiCharacters); //-20° N / +1° E
				string sD3 = geographicalPoint.ToString(AngleFormat.sD, 3, separator, forceAsciiCharacters); //-20,499° N / +0,998° E

				string sDD0 = geographicalPoint.ToString(AngleFormat.sDD, 0, separator, forceAsciiCharacters); //-20° N / +01° E
				string sDD3 = geographicalPoint.ToString(AngleFormat.sDD, 3, separator, forceAsciiCharacters); //-20,499° N / +00,998° E

				string sDM0 = geographicalPoint.ToString(AngleFormat.sDM, 0, separator, forceAsciiCharacters); //-20° 30′ N / +1° 0′ E
				string sDM3 = geographicalPoint.ToString(AngleFormat.sDM, 3, separator, forceAsciiCharacters); //-20° 29,922′ N / +0° 59,856′ E

				string sDDMM0 = geographicalPoint.ToString(AngleFormat.sDDMM, 0, separator, forceAsciiCharacters); //-20° 30′ N / +01° 00′ E
				string sDDMM3 = geographicalPoint.ToString(AngleFormat.sDDMM, 3, separator, forceAsciiCharacters); //-20° 29,922′ N / +00° 59,856′ E

				string sDMS0 = geographicalPoint.ToString(AngleFormat.sDMS, 0, separator, forceAsciiCharacters); //-20° 29′ 55'' N / +0° 59′ 51'' E
				string sDMS3 = geographicalPoint.ToString(AngleFormat.sDMS, 3, separator, forceAsciiCharacters); //-20° 29′ 55,320'' N / +0° 59′ 51,360'' E

				string sDDMMSS0 = geographicalPoint.ToString(AngleFormat.sDDMMSS, 0, separator, forceAsciiCharacters); //-20° 29′ 55'' N / +00° 59′ 51'' E
				string sDDMMSS3 = geographicalPoint.ToString(AngleFormat.sDDMMSS, 3, separator, forceAsciiCharacters); //-20° 29′ 55,320'' N / +00° 59′ 51,360'' E
			}

			//Retreiving a 'Maidenhead Locator' for a certain precision
			{
				//Transmitter mast 'Haardtkopf'
				GeographicalPoint geographicalPoint = new GeographicalPoint(
					false, 49, 50, 44.2,
					false, 7, 3, 14.8
				);

				//via Method
				{
					string precision1 = geographicalPoint.GetMaidenheadLocator(true, 1); //JN
					string precision2 = geographicalPoint.GetMaidenheadLocator(true, 2); //JN39
					string precision3 = geographicalPoint.GetMaidenheadLocator(true, 3); //JN39mu
					string precision4 = geographicalPoint.GetMaidenheadLocator(true, 4); //JN39mu62

					//Small letters for subsquares?
					string precision5a = geographicalPoint.GetMaidenheadLocator(true, 5); //JN39mu62lw
					string precision5b = geographicalPoint.GetMaidenheadLocator(false, 5); //JN39MU62LW

					//Invalid precisions will be tolerated
					string invalidPrecision1 = geographicalPoint.GetMaidenheadLocator(true, -1); //JN
					string invalidPrecision2 = geographicalPoint.GetMaidenheadLocator(true, 100); //JN39mu62lw87je58xa
				}

				//via Index
				{
					int minIndex = MaidenheadLocator.MinPrecision - 1;
					int maxIndex = MaidenheadLocator.MaxPrecision - 1;

					//index immediately
					string indexTest1 = geographicalPoint.GetMaidenheadLocators(true)[3]; //JN39mu62

					//invalid index -> ArgumentOutOfRangeException!
					string indexTest2 = geographicalPoint.GetMaidenheadLocators(true)[100];

					//index safely
					string indexTest3 = geographicalPoint.GetMaidenheadLocators(true)[Math.Min(maxIndex, Math.Max(minIndex, -100))]; //JN
					string indexTest4 = geographicalPoint.GetMaidenheadLocators(true)[Math.Min(maxIndex, Math.Max(minIndex, 100))]; //JN39mu62lw87je58xa
				}
			}

			//Retreiving geographical coordinates by a 'Maidenhead Locator'
			{
				GeographicalPoint source = new GeographicalPoint(0, 0);
				double latitude;
				double longitude;

				//Precision 2
				{
					string sourceLocator = source.GetMaidenheadLocator(true, 2);

					//Latitude = 0.0; Longitude = 0.0
					MaidenheadLocator.GeographicalCoordinatesByMaidenheadLocator(
						sourceLocator,
						PositionInRectangle.BottomLeft, //'BottomLeft' is the position by 'Maidenhead Locator' definitions
						out latitude,
						out longitude
					);

					//Latitude = 0.5; Longitude = 1.0
					MaidenheadLocator.GeographicalCoordinatesByMaidenheadLocator(
						sourceLocator,
						PositionInRectangle.MiddleMiddle,
						out latitude,
						out longitude
					);

					//Latitude = 1.0; Longitude = 2.0
					MaidenheadLocator.GeographicalCoordinatesByMaidenheadLocator(
						sourceLocator,
						PositionInRectangle.TopRight,
						out latitude,
						out longitude
					);
				}

				//Precision 4
				{
					string sourceLocator = source.GetMaidenheadLocator(true, 4);

					//Latitude = 0.0; Longitude = 0.0
					MaidenheadLocator.GeographicalCoordinatesByMaidenheadLocator(
						sourceLocator,
						PositionInRectangle.BottomLeft,
						out latitude,
						out longitude
					);

					//Latitude = 0.0020833333333333333; Longitude = 0.0041666666666666666
					MaidenheadLocator.GeographicalCoordinatesByMaidenheadLocator(
						sourceLocator,
						PositionInRectangle.MiddleMiddle,
						out latitude,
						out longitude
					);

					//Latitude = 0.0041666666666666666; Longitude = 0.0083333333333333332
					MaidenheadLocator.GeographicalCoordinatesByMaidenheadLocator(
						sourceLocator,
						PositionInRectangle.TopRight,
						out latitude,
						out longitude
					);
				}
			}

			//Position in a square
			{
				string sourceLocator = "JN";

				//Latitude = 50.0; Longitude = 0.0
				GeographicalPoint topLeftInSquare = new GeographicalPoint(sourceLocator, PositionInRectangle.TopLeft);

				//Latitude = 45.0; Longitude = 20.0
				GeographicalPoint middleRightInSquare = new GeographicalPoint(sourceLocator, PositionInRectangle.MiddleRight);

				//Latitude = 40.0; Longitude = 10.0
				GeographicalPoint bottomMiddleInSquare = new GeographicalPoint(sourceLocator, PositionInRectangle.BottomMiddle);

				//Latitude = 40.0; Longitude = 0.0
				GeographicalPoint bottomLeftInSquare = new GeographicalPoint(sourceLocator, PositionInRectangle.BottomLeft);

				double distance1 = bottomLeftInSquare.GetDistance(sourceLocator, PositionInRectangle.BottomLeft);
				//0.0

				double distance2 = bottomLeftInSquare.GetDistance(sourceLocator, PositionInRectangle.TopRight);
				//1916.1304193647493

				double azimuth1 = bottomLeftInSquare.GetAzimuth(sourceLocator, PositionInRectangle.TopRight);
				//47.911573690505122

				double azimuth2 = bottomLeftInSquare.GetAzimuth(sourceLocator, PositionInRectangle.TopLeft);
				//0.0

				//Approximation towards 90 degrees
				{
					PositionInRectangle bottomLeft = PositionInRectangle.BottomLeft;
					PositionInRectangle bottomRight = PositionInRectangle.BottomRight;

					double azimuth3_1 = new GeographicalPoint("JN", bottomLeft).GetAzimuth("JN", bottomRight);
					//83.533645605725511

					double azimuth3_2 = new GeographicalPoint("JN39", bottomLeft).GetAzimuth("JN39", bottomRight);
					//89.245257435506943

					double azimuth3_3 = new GeographicalPoint("JN39ml", bottomLeft).GetAzimuth("JN39ml", bottomRight);
					//89.968336102950133

					double azimuth3_4 = new GeographicalPoint("JN39ml36", bottomLeft).GetAzimuth("JN39ml36", bottomRight);
					//89.9968324290917

					double azimuth3_5 = new GeographicalPoint("JN39ml36oi", bottomLeft).GetAzimuth("JN39ml36oi", bottomRight);
					//89.999868012822745

					double azimuth3_9 = new GeographicalPoint("JN39ml36oi72oi72oi", bottomLeft).GetAzimuth("JN39ml36oi72oi72oi", bottomRight);
					//90.0
				}
			}

			//Neighbouring squares
			{
				GeographicalPoint bottomLeft = new GeographicalPoint("JN39IP", PositionInRectangle.BottomLeft);

				{
					GeographicalPoint topRight = new GeographicalPoint("JN39HO", PositionInRectangle.TopRight);

					double deltaLatitude = bottomLeft.Latitude - topRight.Latitude; //0.0
					double deltaLongitude = bottomLeft.Longitude - topRight.Longitude; //0.00000000000000088817841970012523
				}

				{
					GeographicalPoint bottomRight = new GeographicalPoint("JN39HP", PositionInRectangle.BottomRight);

					double deltaLatitude = bottomLeft.Latitude - bottomRight.Latitude; //0.0
					double deltaLongitude = bottomLeft.Longitude - bottomRight.Longitude; //0.00000000000000088817841970012523
				}

				{
					GeographicalPoint leftBottomLeft = new GeographicalPoint("JN39HP", PositionInRectangle.BottomLeft);

					double deltaLatitude = bottomLeft.Latitude - leftBottomLeft.Latitude; //0.0
					double deltaLongitude = bottomLeft.Longitude - leftBottomLeft.Longitude; //0.083333333333333925 (Grid width for precision 3)
				}

				{
					GeographicalPoint aboveBottomLeft = new GeographicalPoint("JN39IQ", PositionInRectangle.BottomLeft);

					double deltaLatitude = bottomLeft.Latitude - aboveBottomLeft.Latitude; //-0.0416666666666643 (Grid height for precision 3)
					double deltaLongitude = bottomLeft.Longitude - aboveBottomLeft.Longitude; //0.0
				}
			}

			//Waypoint projections
			{
				/*
				 * Example:
				 * 
				 * I'm on the observation tower 'Erbeskopf'.
				 * 
				 * I see a transmitter mast with azimuth 349 degrees.
				 * I assume a distance of 13 km.
				 * 
				 * What are its geographical coodinates and what is the object I see?
				 * */

				GeographicalPoint observationTowerErbeskopf = new GeographicalPoint(
					false, 49, 43, 45.28,
					false, 7, 05, 22.65
				);

				GeographicalPoint waypointProjection = observationTowerErbeskopf.GetWaypointProjection(
					349, 13
				);

				//+49° 50′ 38'' N / +07° 03′ 18'' E
				string geographicalCoodinates = waypointProjection.ToString(AngleFormat.sDDMMSS, 0, " / ", false);

				/*
				 * I look for a appropriate object (transmitter mast) in the vicinity of that geographical position on a map.
				 * 
				 * I find the transmitter mast 'Haardtkopf'.
				 * */
			}
		}
	}
}